import {throwIfAppliedToNonPrototypeProperty, getPolymerObserversArray, getPolymerPropertyObject} from './utils';

export function observe(observedProps) {
  if (observedProps.indexOf(',') > 0 || observedProps.indexOf('.') > 0) {
    // observing multiple properties or path
    return (target, functionKey) => {
      throwIfAppliedToNonPrototypeProperty(target, 'observe', functionKey);

      const klass = target.constructor;

      const observers = getPolymerObserversArray(klass);

      observers.push(`${functionKey}(${observedProps})`);
    };
  } else {
    // observing single property
    return (target, functionKey) => {
      throwIfAppliedToNonPrototypeProperty(target, 'observe', functionKey);

      const klass = target.constructor;

      const propertyObject = getPolymerPropertyObject(klass, observedProps);

      if (propertyObject.observer) {
        throw new Error(`@observe decorator: property ${observedProps} already has assigned observer: ${propertyObject.observer}`)
      }

      propertyObject.observer = functionKey;
    };
  }
}
