import {throwIfAppliedToNonPrototypeProperty, getPolymerPropertyObject} from './utils';

const checkValue = (value) =>
value !== undefined && value !== null;

const getCheckValueFn = (targetFnName) => function (...args) {
  if (!checkValue(args[0])) {
    return;
  }
  this[targetFnName].apply(this, args);
}

export function observeOnValue(observedProps) {
  if (observedProps.indexOf('.') > 0) {
    throw new Error('@observeOnValue decorator does not support observing a path')
  }

  if (observedProps.indexOf(',') > 0) {
    throw new Error('@observeOnValue decorator does not support observing multiple properties')
  }

  return (target, functionKey) => {
    throwIfAppliedToNonPrototypeProperty(target, 'observeOnValue', functionKey);

    const klass = target.constructor;

    const propertyObject = getPolymerPropertyObject(klass, observedProps);

    if (propertyObject.observer) {
      throw new Error(`@observeOnValue decorator: property ${observedProps} already has assigned observer: ${propertyObject.observer}`)
    }

    const observeFunctionName = `__observe_${functionKey}_on_value`;

    target[observeFunctionName] = getCheckValueFn(functionKey);

    propertyObject.observer = observeFunctionName;
  };

}
