import {throwIfAppliedToNonPrototypeProperty, getPolymerPropertyObject} from './utils';

const getCheckOldValueFn = (targetFnName) => function (...args) {
  const fn = this[targetFnName];
  if (!fn.__executed && args[1] === undefined) {
    return;
  }
  fn.__executed = true;
  fn.apply(this, args);
}

export function observeSinceValue(observedProps) {
  if (observedProps.indexOf('.') > 0) {
    throw new Error('@observeSinceValue decorator does not support observing a path')
  }

  if (observedProps.indexOf(',') > 0) {
    throw new Error('@observeSinceValue decorator does not support observing multiple properties')
  }

  return (target, functionKey) => {
    throwIfAppliedToNonPrototypeProperty(target, 'observeSinceValue', functionKey);

    const klass = target.constructor;

    const propertyObject = getPolymerPropertyObject(klass, observedProps);

    if (propertyObject.observer) {
      throw new Error(`@observeSinceValue decorator: property ${observedProps} already has assigned observer: ${propertyObject.observer}`)
    }

    const observeFunctionName = `__observe_${functionKey}_on_value`;

    target[observeFunctionName] = getCheckOldValueFn(functionKey);

    propertyObject.observer = observeFunctionName;
  };

}
