const nonStaticPropertyErrorTemplate = (decoratorName, propertyKey) =>
  `@${decoratorName} decorator must be applied to a static property: ${propertyKey}`;

const nonPrototypePropertyErrorTemplate = (decoratorName, propertyKey) =>
  `@${decoratorName} decorator must be applied to a prototype property: ${propertyKey}`;

export function throwIfAppliedToNonStaticProperty(target, decoratorName, propertyKey) {
  if (target instanceof HTMLElement) {
    throw new Error(nonStaticPropertyErrorTemplate(decoratorName, propertyKey));
  }
}

export function throwIfAppliedToNonPrototypeProperty(target, decoratorName, propertyKey) {
  if (!(target instanceof HTMLElement)) {
    throw new Error(nonPrototypePropertyErrorTemplate(decoratorName, propertyKey));
  }
}

export function getPolymerPropertiesObject(klass) {
  if (!klass.hasOwnProperty('properties')) {
    klass.properties = {};
  }
  return klass.properties;
}

export function getPolymerObserversArray(klass) {
  if (!klass.hasOwnProperty('observers')) {
    klass.observers = [];
  }
  return klass.observers;
}

export function getPolymerPropertyObject(klass, propertyKey) {
  const properties = getPolymerPropertiesObject(klass);

  properties[propertyKey] = properties[propertyKey] || {};

  return klass.properties[propertyKey];
}
